/*
	NSFont.h
	Application Kit
	Copyright (c) 1994-1996, NeXT Software, Inc.
	All rights reserved.
*/

#import <Foundation/Foundation.h>
#import <AppKit/NSFontManager.h>
#import <AppKit/AppKitDefines.h>

typedef unsigned int NSGlyph;

enum {
    NSControlGlyph = 0x00FFFFFF,
    NSNullGlyph = 0x0
};

typedef enum _NSGlyphRelation {
    NSGlyphBelow = 1,
    NSGlyphAbove = 2,
} NSGlyphRelation;

APPKIT_EXTERN const float *NSFontIdentityMatrix;

@interface NSFont : NSObject <NSCopying, NSCoding> {
@private
    NSString *_name;
    float _size;
    int _reservedFont1;
    float *_matrix;
    int _fontNum;
    struct _NSFaceInfo *_faceInfo;
    id _otherFont;
    struct __fFlags {
      /* the following instance variables are no longer part of the API */
	unsigned int usedByWS:1;
	unsigned int usedByPrinter:1;
	unsigned int isScreenFont:1;
	unsigned int _systemFontType:4;
	unsigned int _flippedUsedByWS:1;
	unsigned int _flippedUsedByPrinter:1;
	unsigned int _RESERVED:4;
	unsigned int _matrixIsIdentity:1;
	unsigned int _matrixIsFlipped:1;
	unsigned int _hasStyle:1;
    } _fFlags;
    unsigned short _reservedFont2;
    int _flippedFontNum;
}

+ (NSFont *)fontWithName:(NSString *)fontName size:(float)fontSize;
+ (NSFont *)fontWithName:(NSString *)fontName matrix:(const float *)fontMatrix;
+ (void)useFont:(NSString *)fontName;

+ (NSFont *)userFontOfSize:(float)fontSize;
+ (NSFont *)userFixedPitchFontOfSize:(float)fontSize;
+ (NSFont *)systemFontOfSize:(float)fontSize;
+ (NSFont *)boldSystemFontOfSize:(float)fontSize;
+ (void)setUserFont:(NSFont *)aFont;
+ (void)setUserFixedPitchFont:(NSFont *)aFont;

#ifndef STRICT_OPENSTEP

+ (NSFont *)titleBarFontOfSize:(float)fontSize;
+ (NSFont *)menuFontOfSize:(float)fontSize;
+ (NSFont *)messageFontOfSize:(float)fontSize;
+ (NSFont *)paletteFontOfSize:(float)fontSize;
+ (NSFont *)toolTipsFontOfSize:(float)fontSize;

#endif

- (float)pointSize;
- (NSString *)fontName;
- (NSString *)familyName;
- (NSString *)displayName;

- (const float *)matrix;
- (void)set;
- (float)widthOfString:(NSString *)string;
- (NSFont *)screenFont;
- (NSFont *)printerFont;

- (BOOL)isBaseFont;

/* The "widths" method returns a pointer to an array of 256 widths.  They are unscaled but are pre-divided by 1000.  All other methods return scaled numbers.  If the font was created with a matrix, the matrix is applied automatically; otherwise the coordinates are multiplied by size.
*/

- (float *)widths;
- (NSRect)boundingRectForFont;
- (NSRect)boundingRectForGlyph:(NSGlyph)aGlyph;
- (NSSize)advancementForGlyph:(NSGlyph)ag;
- (NSDictionary *)afmDictionary;
- (NSString *)afmFileContents;
- (BOOL)glyphIsEncoded:(NSGlyph)aGlyph;

#ifndef STRICT_OPENSTEP
+ (NSArray *)preferredFontNames;
+ (void)setPreferredFontNames:(NSArray *)fontNameArray;

- (NSSize) maximumAdvancement; /* returns widest advancement, or constant advancement if the font is fixed-pitch. */
- (NSStringEncoding) mostCompatibleStringEncoding;
#endif

- (float) underlinePosition;	/* was fontUnderlinePosition */
- (float) underlineThickness;	/* was fontUnderlineThickness */
- (float) italicAngle;	/* was fontItalicAngle */
- (float) capHeight;	/* used for translating vertical baselines */
- (float) xHeight;
- (float) ascender;
- (float) descender;
- (BOOL) isFixedPitch;
- (NSString *) encodingScheme;
- (NSGlyph) glyphWithName:(NSString *)aName;

/* Glyph related methods.

The position of a glyph is where you need to be to show the glyph. The position is relative to the position of the other glyph supplied as argument. If other glyph is 0 (no glyph), then this reduces to being relative to the origin.

The optional isNominal argument indicates whether the returned position is the standard advancement specified for the previous glyph (ie, width from the AFM file). It could be NO if the two glyphs are kerned, for instance.

The optional metricsExist argument specifies whether the metrics for the combination were found; if not, the returned point will be computed using the bounding boxes of the glyphs and may not be very pretty.

*/

/* Position two base glyphs next to each other.  In this method, 0 (i.e., no glyph) may be specified for either glyph.
*/
- (NSPoint)positionOfGlyph:(NSGlyph)curGlyph precededByGlyph:(NSGlyph)prevGlyph isNominal:(BOOL *)nominal;

#ifndef STRICT_OPENSTEP
/* Given a base glyph followed by a bunch of "overstruck" glyphs, see if there's a corresponding set of metrics in the font.  If so, return the NUMBER of matching glyphs from the group and fill in "points" with the positions, relative to (0,0) for the base glyph.  Partial sequences may be returned.  If the return value is less than numGlyphs, then the leftover glyphs can be rendered via positionOfGlyph:forCharacter:struckOverRect:, using the union-rect of the glyphs for which the font had metrics.
*/
- (int) positionsForCompositeSequence:(NSGlyph *)someGlyphs numberOfGlyphs:(int)numGlyphs pointArray:(NSPoint *)points;

/* Basic base/accent positioning.  Try to position the glyphs, using metric information if it exists.  If metrics do not exist, this method may defer to "positionOfGlyph:struckOverRect:metricsExist:" for the answer.  In any case, if metrics do not exist, the answer may be invalid.
*/
- (NSPoint)positionOfGlyph:(NSGlyph)curGlyph struckOverGlyph:(NSGlyph)prevGlyph metricsExist:(BOOL *)exist;

/* Base/accent positioning when a previous glyph (i.e., aRect) may not be in the same font or has no metrics -- use its rectangle.  Try to position the glyphs, using generalized metric information if it exists.  If metrics do not exist, the result may not be valid.
*/
- (NSPoint)positionOfGlyph:(NSGlyph)aGlyph struckOverRect:(NSRect)aRect metricsExist:(BOOL *)exist;

/* As a last resort, this attempts to do positioning of base/accent based on semantic knowledge of various "accent" characters.  It is NOT guaranteed to be pretty, but works in the absence of proper metrics for "many" diacritical marks in the Latin, Greek, and Cyrillic scripts.  The caller asserts that "aGlyph" is a rendering in the font for the Unicode character "aChar".  The rectangle is presumed to be the rectangle of some other glyph, whether known in this font or not.  The returned position tells where to put "aGlyph" in relation to "aRect"; i.e., presuming aRect is shown at {0,0} one may "moveto" the returned position and show "aGlyph".
*/
- (NSPoint)positionOfGlyph:(NSGlyph)aGlyph forCharacter:(unichar)aChar struckOverRect:(NSRect)aRect;

/* This is for specific types of positioning above/below (mainly for Indic support).  Metric support is usually required for the result to be pretty.  The default is to join glyphs without gaps and return the advancement of baseGlyph.
*/
- (NSPoint)positionOfGlyph:(NSGlyph)thisGlyph withRelation:(NSGlyphRelation)rel toBaseGlyph:(NSGlyph)baseGlyph totalAdvancement:(NSSize *)adv metricsExist:(BOOL *)exist;

#endif

@end

/* These are the strings that are available in the "afmDictionary".  For other things, use "afmFileContents".  The floating point values (e.g., ascender, cap height) can be converted to floating point via NSString's "floatValue" method.
*/
APPKIT_EXTERN NSString *NSAFMFamilyName;
APPKIT_EXTERN NSString *NSAFMFontName;
APPKIT_EXTERN NSString *NSAFMFormatVersion;
APPKIT_EXTERN NSString *NSAFMFullName;
APPKIT_EXTERN NSString *NSAFMNotice;
APPKIT_EXTERN NSString *NSAFMVersion;
APPKIT_EXTERN NSString *NSAFMWeight;
APPKIT_EXTERN NSString *NSAFMEncodingScheme;
APPKIT_EXTERN NSString *NSAFMCharacterSet;

APPKIT_EXTERN NSString *NSAFMCapHeight;
APPKIT_EXTERN NSString *NSAFMXHeight;
APPKIT_EXTERN NSString *NSAFMAscender;
APPKIT_EXTERN NSString *NSAFMDescender;
APPKIT_EXTERN NSString *NSAFMUnderlinePosition;
APPKIT_EXTERN NSString *NSAFMUnderlineThickness;
APPKIT_EXTERN NSString *NSAFMItalicAngle;
APPKIT_EXTERN NSString *NSAFMMappingScheme;

